"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.onConnect = onConnect;
exports.onDisconnect = onDisconnect;
Object.defineProperty(exports, "clientCommands", {
  enumerable: true,
  get: function () {
    return _commands.clientCommands;
  }
});
loader.lazyRequireGetter(this, "_commands", "devtools/client/debugger/src/client/firefox/commands");
loader.lazyRequireGetter(this, "_create", "devtools/client/debugger/src/client/firefox/create");
loader.lazyRequireGetter(this, "_prefs", "devtools/client/debugger/src/utils/prefs");
loader.lazyRequireGetter(this, "_telemetry", "devtools/client/debugger/src/utils/telemetry");

var _sourceQueue = _interopRequireDefault(require("../utils/source-queue"));

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

/* This Source Code Form is subject to the terms of the Mozilla Public
 * License, v. 2.0. If a copy of the MPL was not distributed with this
 * file, You can obtain one at <http://mozilla.org/MPL/2.0/>. */
let actions;
let targetList;
let resourceWatcher;

async function onConnect(devToolsClient, _targetList, _resourceWatcher, _actions, store) {
  actions = _actions;
  targetList = _targetList;
  resourceWatcher = _resourceWatcher;
  (0, _commands.setupCommands)({
    devToolsClient,
    targetList
  });
  (0, _create.setupCreate)({
    store
  });

  _sourceQueue.default.initialize(actions);

  const {
    targetFront
  } = targetList;

  if (targetFront.isBrowsingContext || targetFront.isParentProcess) {
    targetList.listenForWorkers = true;

    if (targetFront.localTab && _prefs.features.windowlessServiceWorkers) {
      targetList.listenForServiceWorkers = true;
      targetList.destroyServiceWorkersOnNavigation = true;
    }

    await targetList.startListening();
  }

  await targetList.watchTargets(targetList.ALL_TYPES, onTargetAvailable, onTargetDestroyed); // Use independant listeners for SOURCE and THREAD_STATE in order to ease
  // doing batching and notify about a set of SOURCE's in one redux action.

  await resourceWatcher.watchResources([resourceWatcher.TYPES.SOURCE], {
    onAvailable: onSourceAvailable
  });
  await resourceWatcher.watchResources([resourceWatcher.TYPES.THREAD_STATE], {
    onAvailable: onBreakpointAvailable
  });
  await resourceWatcher.watchResources([resourceWatcher.TYPES.ERROR_MESSAGE], {
    onAvailable: actions.addExceptionFromResources
  });
}

function onDisconnect() {
  targetList.unwatchTargets(targetList.ALL_TYPES, onTargetAvailable, onTargetDestroyed);
  resourceWatcher.unwatchResources([resourceWatcher.TYPES.SOURCE], {
    onAvailable: onSourceAvailable
  });
  resourceWatcher.unwatchResources([resourceWatcher.TYPES.THREAD_STATE], {
    onAvailable: onBreakpointAvailable
  });
  resourceWatcher.unwatchResources([resourceWatcher.TYPES.ERROR_MESSAGE], {
    onAvailable: actions.addExceptionFromResources
  });

  _sourceQueue.default.clear();
}

async function onTargetAvailable({
  targetFront,
  isTargetSwitching
}) {
  const isBrowserToolbox = targetList.targetFront.isParentProcess;
  const isNonTopLevelFrameTarget = !targetFront.isTopLevel && targetFront.targetType === targetList.TYPES.FRAME;

  if (isBrowserToolbox && isNonTopLevelFrameTarget) {
    // In the BrowserToolbox, non-top-level frame targets are already
    // debugged via content-process targets.
    // Do not attach the thread here, as it was already done by the
    // corresponding content-process target.
    return;
  }

  if (!targetFront.isTopLevel) {
    await actions.addTarget(targetFront);
    return;
  }

  if (isTargetSwitching) {
    // Simulate navigation actions when target switching.
    // The will-navigate event will be missed when using target switching,
    // however `navigate` corresponds more or less to the load event, so it
    // should still be received on the new target.
    actions.willNavigate({
      url: targetFront.url
    });
  } // At this point, we expect the target and its thread to be attached.


  const {
    threadFront
  } = targetFront;

  if (!threadFront) {
    console.error("The thread for", targetFront, "isn't attached.");
    return;
  }

  targetFront.on("will-navigate", actions.willNavigate);
  targetFront.on("navigate", actions.navigated);
  await threadFront.reconfigure({
    observeAsmJS: true,
    pauseWorkersUntilAttach: true,
    skipBreakpoints: _prefs.prefs.skipPausing,
    logEventBreakpoints: _prefs.prefs.logEventBreakpoints
  }); // Retrieve possible event listener breakpoints

  actions.getEventListenerBreakpointTypes().catch(e => console.error(e)); // Initialize the event breakpoints on the thread up front so that
  // they are active once attached.

  actions.addEventListenerBreakpoints([]).catch(e => console.error(e));
  await actions.connect(targetFront.url, threadFront.actor, targetFront.isWebExtension);
  await actions.addTarget(targetFront);
}

function onTargetDestroyed({
  targetFront
}) {
  if (targetFront.isTopLevel) {
    targetFront.off("will-navigate", actions.willNavigate);
    targetFront.off("navigate", actions.navigated);
  }

  actions.removeTarget(targetFront);
}

async function onSourceAvailable(sources) {
  const frontendSources = await Promise.all(sources.map(async source => {
    const threadFront = await source.targetFront.getFront("thread");
    const frontendSource = (0, _create.prepareSourcePayload)(threadFront, source);
    return frontendSource;
  }));
  await actions.newGeneratedSources(frontendSources);
}

async function onBreakpointAvailable(breakpoints) {
  for (const resource of breakpoints) {
    const threadFront = await resource.targetFront.getFront("thread");

    if (resource.state == "paused") {
      const pause = await (0, _create.createPause)(threadFront.actor, resource);
      actions.paused(pause);
      (0, _telemetry.recordEvent)("pause", {
        reason: resource.why.type
      });
    } else if (resource.state == "resumed") {
      actions.resumed(threadFront.actorID);
    }
  }
}